// (C) Copyright 2002-2007 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

//-----------------------------------------------------------------------------
//----- CExtQRCode.cpp : Implementation of CExtQRCode
//-----------------------------------------------------------------------------
#include "StdAfx.h"
#include "qrencode.h"
#include "atlexcept.h"
#include "atlconv.h"



constexpr auto CExtQRCodeProxyFlags =
AcDbProxyEntity::kEraseAllowed |
AcDbProxyEntity::kTransformAllowed |
AcDbProxyEntity::kColorChangeAllowed |
AcDbProxyEntity::kLayerChangeAllowed |
AcDbProxyEntity::kLinetypeChangeAllowed |
AcDbProxyEntity::kLinetypeScaleChangeAllowed |
AcDbProxyEntity::kVisibilityChangeAllowed |
AcDbProxyEntity::kCloningAllowed |
AcDbProxyEntity::kLineWeightChangeAllowed |
AcDbProxyEntity::kPlotStyleNameChangeAllowed |
AcDbProxyEntity::kMaterialChangeAllowed;

//-----------------------------------------------------------------------------
Adesk::UInt32 CExtQRCode::kCurrentVersionNumber =1 ;

struct QRcodeDeleter
{
    void operator()(QRcode *p) { QRcode_free(p); }
};

//-----------------------------------------------------------------------------
ACRX_DXF_DEFINE_MEMBERS (
	CExtQRCode, AcDbEntity,
	AcDb::kDHL_CURRENT, AcDb::kMReleaseCurrent, 
    CExtQRCodeProxyFlags, EXTQRCODE,
QRCODE
|Product Desc:     A QRcode entity for CAD
|Company:          CadExt
|WEB Address:      www.cadext.com 
)

//-----------------------------------------------------------------------------
CExtQRCode::CExtQRCode () : AcDbEntity () 
{
    this->setDefaults();
}

CExtQRCode::~CExtQRCode ()
{
}

void CExtQRCode::setDefaults()
{
    m_position = AcGePoint3d::kOrigin;
    m_normal = AcGeVector3d::kZAxis;
    m_direction = AcGeVector3d::kXAxis;
    m_scale = 1.0;
    m_margin = 4;
    m_invert = false;
    m_caseSensitive = true;
    m_contents.setEmpty();
    m_encoding = kQR_MODE_8;
    m_ecLevel = kQR_ECLEVEL_L;
    //no filer;
    m_version = -1;
}

//-----------------------------------------------------------------------------
//----- AcDbObject protocols
//- Dwg Filing protocol
Acad::ErrorStatus CExtQRCode::dwgOutFields (AcDbDwgFiler *pFiler) const {
	assertReadEnabled () ;
	//----- Save parent class information first.
	Acad::ErrorStatus es =AcDbEntity::dwgOutFields (pFiler) ;
	if ( es != Acad::eOk )
		return (es) ;
	//----- Object version number needs to be saved first
	if ( (es =pFiler->writeUInt32 (CExtQRCode::kCurrentVersionNumber)) != Acad::eOk )
		return (es) ;
	//----- Output params
	//.....

    pFiler->writePoint3d(m_position);
    pFiler->writeVector3d(m_normal);
    pFiler->writeVector3d(m_direction);
    pFiler->writeDouble(m_scale);

    Adesk::UInt32 margin = m_margin;
    pFiler->writeUInt32(margin);

    pFiler->writeBool(m_invert);
    pFiler->writeBool(m_caseSensitive);
    pFiler->writeString(m_contents);

    Adesk::Int32 encoding = m_encoding;
    pFiler->writeInt32(encoding);

    Adesk::Int32 ec = m_ecLevel;
    pFiler->writeInt32(ec);

	return (pFiler->filerStatus ()) ;
}

Acad::ErrorStatus CExtQRCode::dwgInFields (AcDbDwgFiler *pFiler) {
	assertWriteEnabled () ;
    this->setDefaults();
	//----- Read parent class information first.
	Acad::ErrorStatus es =AcDbEntity::dwgInFields (pFiler) ;
	if ( es != Acad::eOk )
		return (es) ;
	//----- Object version number needs to be read first
	Adesk::UInt32 version =0 ;
	if ( (es =pFiler->readUInt32 (&version)) != Acad::eOk )
		return (es) ;
	if ( version > CExtQRCode::kCurrentVersionNumber )
		return (Acad::eMakeMeProxy) ;
	//- Uncomment the 2 following lines if your current object implementation cannot
	//- support previous version of that object.
	//if ( version < CExtQRCode::kCurrentVersionNumber )
	//	return (Acad::eMakeMeProxy) ;
	//----- Read params
	//.....

    pFiler->readPoint3d(&m_position);
    pFiler->readVector3d(&m_normal);
    pFiler->readVector3d(&m_direction);
    pFiler->readDouble(&m_scale);

    Adesk::UInt32 margin;
    pFiler->readUInt32(&margin);
    m_margin = margin;

    pFiler->readBool(&m_invert);
    pFiler->readBool(&m_caseSensitive);
    pFiler->readString(m_contents);

    Adesk::Int32 encoding;
    pFiler->readInt32(&encoding);
    m_encoding = (CExtQRCode::QRModeEncoding)encoding;

    Adesk::Int32 ec;
    pFiler->readInt32(&ec);
    m_ecLevel= (CExtQRCode::QRECLevel)ec;

	return (pFiler->filerStatus ()) ;
}

//- Dxf Filing protocol
Acad::ErrorStatus CExtQRCode::dxfOutFields (AcDbDxfFiler *pFiler) const {
	assertReadEnabled () ;
	//----- Save parent class information first.
	Acad::ErrorStatus es =AcDbEntity::dxfOutFields (pFiler) ;
	if ( es != Acad::eOk )
		return (es) ;
	es =pFiler->writeItem (AcDb::kDxfSubclass, _RXST("CExtQRCode")) ;
	if ( es != Acad::eOk )
		return (es) ;
	//----- Object version number needs to be saved first
	if ( (es =pFiler->writeUInt32 (kDxfInt32, CExtQRCode::kCurrentVersionNumber)) != Acad::eOk )
		return (es) ;
	//----- Output params
	//.....

	return (pFiler->filerStatus ()) ;
}

Acad::ErrorStatus CExtQRCode::dxfInFields (AcDbDxfFiler *pFiler) {
	assertWriteEnabled () ;
	//----- Read parent class information first.
	Acad::ErrorStatus es =AcDbEntity::dxfInFields (pFiler) ;
	if ( es != Acad::eOk || !pFiler->atSubclassData (_RXST("CExtQRCode")) )
		return (pFiler->filerStatus ()) ;
	//----- Object version number needs to be read first
	struct resbuf rb ;
	pFiler->readItem (&rb) ;
	if ( rb.restype != AcDb::kDxfInt32 ) {
		pFiler->pushBackItem () ;
		pFiler->setError (Acad::eInvalidDxfCode, _RXST("\nError: expected group code %d (version #)"), AcDb::kDxfInt32) ;
		return (pFiler->filerStatus ()) ;
	}
	Adesk::UInt32 version =(Adesk::UInt32)rb.resval.rlong ;
	if ( version > CExtQRCode::kCurrentVersionNumber )
		return (Acad::eMakeMeProxy) ;
	//- Uncomment the 2 following lines if your current object implementation cannot
	//- support previous version of that object.
	//if ( version < CExtQRCode::kCurrentVersionNumber )
	//	return (Acad::eMakeMeProxy) ;
	//----- Read params in non order dependant manner
	while ( es == Acad::eOk && (es =pFiler->readResBuf (&rb)) == Acad::eOk ) {
		switch ( rb.restype ) {
			//----- Read params by looking at their DXF code (example below)
			//case AcDb::kDxfXCoord:
			//	if ( version == 1 )
			//		cen3d =asPnt3d (rb.resval.rpoint) ;
			//	else 
			//		cen2d =asPnt2d (rb.resval.rpoint) ;
			//	break ;
			//.....

			default:
				//----- An unrecognized group. Push it back so that the subclass can read it again.
				pFiler->pushBackItem () ;
				es =Acad::eEndOfFile ;
				break ;
		}
	}
	//----- At this point the es variable must contain eEndOfFile
	//----- - either from readResBuf() or from pushback. If not,
	//----- it indicates that an error happened and we should
	//----- return immediately.
	if ( es != Acad::eEndOfFile )
		return (Acad::eInvalidResBuf) ;

	return (pFiler->filerStatus ()) ;
}

//- SubXXX() methods (self notification)
Acad::ErrorStatus CExtQRCode::subOpen (AcDb::OpenMode mode) {
	return (AcDbEntity::subOpen (mode)) ;
}

Acad::ErrorStatus CExtQRCode::subErase (Adesk::Boolean erasing) {
	return (AcDbEntity::subErase (erasing)) ;
}

Acad::ErrorStatus CExtQRCode::subCancel () {
	return (AcDbEntity::subCancel ()) ;
}

Acad::ErrorStatus CExtQRCode::subClose () {
	return (AcDbEntity::subClose ()) ;
}

//- Persistent reactor callbacks
void CExtQRCode::openedForModify (const AcDbObject *pDbObj) {
	assertReadEnabled () ;
	AcDbEntity::openedForModify (pDbObj) ;
}

void CExtQRCode::cancelled (const AcDbObject *pDbObj) {
	assertReadEnabled () ;
	AcDbEntity::cancelled (pDbObj) ;
}

void CExtQRCode::objectClosed (const AcDbObjectId objId) {
	assertReadEnabled () ;
	AcDbEntity::objectClosed (objId) ;
}

void CExtQRCode::goodbye (const AcDbObject *pDbObj) {
	assertReadEnabled () ;
	AcDbEntity::goodbye (pDbObj) ;
}

void CExtQRCode::copied (const AcDbObject *pDbObj, const AcDbObject *pNewObj) {
	assertReadEnabled () ;
	AcDbEntity::copied (pDbObj, pNewObj) ;
}

void CExtQRCode::erased (const AcDbObject *pDbObj, Adesk::Boolean bErasing) {
	assertReadEnabled () ;
	AcDbEntity::erased (pDbObj, bErasing) ;
}

void CExtQRCode::modified (const AcDbObject *pDbObj) {
	assertReadEnabled () ;
	AcDbEntity::modified (pDbObj) ;
}

void CExtQRCode::modifiedGraphics (const AcDbEntity *pDbEnt) {
	assertReadEnabled () ;
	AcDbEntity::modifiedGraphics (pDbEnt) ;
}

void CExtQRCode::modifiedXData (const AcDbObject *pDbObj) {
	assertReadEnabled () ;
	AcDbEntity::modifiedXData (pDbObj) ;
}

void CExtQRCode::subObjModified (const AcDbObject *pMainbObj, const AcDbObject *pSubObj) {
	assertReadEnabled () ;
	AcDbEntity::subObjModified (pMainbObj, pSubObj) ;
}

void CExtQRCode::modifyUndone (const AcDbObject *pDbObj) {
	assertReadEnabled () ;
	AcDbEntity::modifyUndone (pDbObj) ;
}

void CExtQRCode::reappended (const AcDbObject *pDbObj) {
	assertReadEnabled () ;
	AcDbEntity::reappended (pDbObj) ;
}

void CExtQRCode::unappended (const AcDbObject *pDbObj) {
	assertReadEnabled () ;
	AcDbEntity::unappended (pDbObj) ;
}

//----- deepClone
Acad::ErrorStatus CExtQRCode::subDeepClone (AcDbObject *pOwnerObject, AcDbObject *&pClonedObject, AcDbIdMapping &idMap, Adesk::Boolean isPrimary) const {
	assertReadEnabled () ;
	return (AcDbEntity::subDeepClone (pOwnerObject, pClonedObject, idMap, isPrimary)) ;
}

//----- wblockClone
Acad::ErrorStatus CExtQRCode::subWblockClone (AcRxObject *pOwnerObject, AcDbObject *&pClonedObject, AcDbIdMapping &idMap, Adesk::Boolean isPrimary) const {
	assertReadEnabled () ;
	return (AcDbEntity::subWblockClone (pOwnerObject, pClonedObject, idMap, isPrimary)) ;
}

//-----------------------------------------------------------------------------
//----- AcDbEntity protocols
Adesk::Boolean CExtQRCode::subWorldDraw (AcGiWorldDraw *mode) 
{
    assertReadEnabled();

    //PerfTimer timer;

    AcGiSubEntityTraits& rTraits = mode->subEntityTraits();
    if(mode->isDragging())
        rTraits.setFillType(kAcGiFillNever);
    else
       rTraits.setFillType(kAcGiFillAlways);

    if(m_contents.isEmpty() || generateQRCode(mode) != eOk)
    {
        generateEmptyQRCode(mode);
    }

    //timer.end();
    return eOk;
}


Adesk::UInt32 CExtQRCode::subSetAttributes (AcGiDrawableTraits *traits) {
	assertReadEnabled () ;
	return (AcDbEntity::subSetAttributes (traits)) ;
}

	//- Osnap points protocol
Acad::ErrorStatus CExtQRCode::subGetOsnapPoints (
	AcDb::OsnapMode osnapMode,
	Adesk::GsMarker gsSelectionMark,
	const AcGePoint3d &pickPoint,
	const AcGePoint3d &lastPoint,
	const AcGeMatrix3d &viewXform,
	AcGePoint3dArray &snapPoints,
	AcDbIntArray &geomIds) const
{
	assertReadEnabled () ;
    switch(osnapMode)
    {
    case AcDb::kOsModeEnd:
        snapPoints.append(m_position);
        break;
    }
	return eOk;
}

Acad::ErrorStatus CExtQRCode::subGetOsnapPoints (
	AcDb::OsnapMode osnapMode,
	Adesk::GsMarker gsSelectionMark,
	const AcGePoint3d &pickPoint,
	const AcGePoint3d &lastPoint,
	const AcGeMatrix3d &viewXform,
	AcGePoint3dArray &snapPoints,
	AcDbIntArray &geomIds,
	const AcGeMatrix3d &insertionMat) const
{
	assertReadEnabled () ;
    switch(osnapMode)
    {
    case AcDb::kOsModeEnd:
        snapPoints.append(m_position);
        break;
    }
    return eOk;
}

//- Grip points protocol
Acad::ErrorStatus CExtQRCode::subGetGripPoints (
	AcGePoint3dArray &gripPoints, AcDbIntArray &osnapModes, AcDbIntArray &geomIds
) const 
{
	assertReadEnabled () ;
    gripPoints.append(m_position);
    return eOk;
}

Acad::ErrorStatus CExtQRCode::subMoveGripPointsAt (const AcDbIntArray &indices, const AcGeVector3d &offset) {
    assertWriteEnabled () ;
    if(indices.length())
    {
        switch(indices[0])
        {
        case 0:
            {
                m_position += offset;
            }
            break;
        }
    }
    return eOk;
}

Acad::ErrorStatus CExtQRCode::subGetGripPoints (
	AcDbGripDataPtrArray &grips, const double curViewUnitSize, const int gripSize, 
	const AcGeVector3d &curViewDir, const int bitflags
) const 
{
	assertReadEnabled () ;
	return eNotImplemented;
}

Acad::ErrorStatus CExtQRCode::subMoveGripPointsAt (
	const AcDbVoidPtrArray &gripAppData, const AcGeVector3d &offset,
	const int bitflags
) 
{
	assertWriteEnabled () ;
    return eNotImplemented;
}

Acad::ErrorStatus CExtQRCode::subTransformBy( const AcGeMatrix3d& xform )
{
    assertWriteEnabled () ;
    m_position.transformBy(xform);
    m_normal.transformBy(xform);
    m_direction.transformBy(xform);
    m_scale *= m_direction.length();
    m_normal.normalize();
    m_direction.normalize();
    return eOk;
}

const AcString& CExtQRCode::contents() const
{
    assertReadEnabled ();
    return m_contents;
}

void CExtQRCode::setContents( const AcString &val )
{
    assertWriteEnabled ();
    m_contents = val;
}

const AcGePoint3d& CExtQRCode::posistion() const
{
     assertReadEnabled ();
     return m_position;
}

void CExtQRCode::setPosistion( const AcGePoint3d &val )
{
    assertWriteEnabled ();
    m_position = val;
}

const AcGeVector3d& CExtQRCode::normal() const
{
    assertReadEnabled ();
    return m_normal; 
}

const AcGeVector3d& CExtQRCode::direction() const
{
    assertReadEnabled ();
    return m_direction;
}

double CExtQRCode::scale() const
{
    assertReadEnabled ();
    return m_scale;
}

void CExtQRCode::setScale( double val )
{
    assertWriteEnabled ();
    m_scale = val;
}

int CExtQRCode::version() const
{
    assertReadEnabled ();
    return m_version;
}

size_t CExtQRCode::margin() const
{
     assertReadEnabled ();
     return m_margin;
}

void CExtQRCode::setMargin( size_t val )
{
    assertWriteEnabled ();
    m_margin = val;
}

bool CExtQRCode::invert() const
{
    assertReadEnabled ();
    return m_invert;
}

void CExtQRCode::setInvert( bool val )
{
    assertWriteEnabled ();
    m_invert = val;
}

bool CExtQRCode::isCaseSensitive() const
{
     assertReadEnabled ();
     return m_caseSensitive;
}

void CExtQRCode::setCaseSensitive( bool val )
{
    assertWriteEnabled ();
    m_caseSensitive = val;
}

CExtQRCode::QRECLevel CExtQRCode::ecLevel() const
{
    assertReadEnabled ();
    return m_ecLevel;
}

void CExtQRCode::setEcLevel( CExtQRCode::QRECLevel val )
{
    assertWriteEnabled ();
    m_ecLevel = val;
}

CExtQRCode::QRModeEncoding CExtQRCode::encoding()
{
    assertReadEnabled ();
    return m_encoding;
}

Acad::ErrorStatus CExtQRCode::generateQRCode(AcGiWorldDraw *mode)
{
    ATL::CW2A contentsUTF8(m_contents,CP_UTF8); //cache this?
    std::unique_ptr<QRcode, QRcodeDeleter> qrcode(QRcode_encodeString(contentsUTF8, 0, (QRecLevel)m_ecLevel, (QRencodeMode)m_encoding, m_caseSensitive ? 1 : 0));
    if(qrcode.get() == NULL)
        return eNullPtr;

    m_version = qrcode->version;
    size_t x = 0, y = 0;
    const size_t realwidth = (qrcode->width + m_margin * 2);

    CQRCodeNode node;
    const double sc = m_scale/realwidth;
    AcGePoint3d refPoint (0.0, realwidth-1,0.0);
    const AcGeVector3d yDir = -1 * m_direction.crossProduct(m_normal).normalize();
    AcGeMatrix3d xform; 
    xform.setCoordSystem(m_position,m_direction*sc,yDir*sc,m_normal*sc);

    AcGiSubEntityTraits& rTraits = mode->subEntityTraits();
    AcGiWorldGeometry& rGeom = mode->geometry();
    rTraits.setTrueColor(node.getOffClr());
    rGeom.pushModelTransform(xform);
    

    for(size_t i= 0; i < m_margin; i++ )
    {
        refPoint.x = 0;
        for(size_t j= 0; j < realwidth; j++ )
        {
            node.draw(rGeom,rTraits,refPoint,m_invert);
            refPoint.x+=1;
        }
        refPoint.y-=1;
    }
    for(y = 0; y < qrcode->width; y++) 
    {
        refPoint.x = 0;
        const unsigned char *row = qrcode->data+(y*qrcode->width);
        for(size_t i= 0; i < m_margin; i++ )
        {
            node.draw(rGeom,rTraits,refPoint, m_invert);
            refPoint.x+=1;
        }
        for(x=0; x<qrcode->width; x++) 
        {
            if(row[x]&0x1) 
                node.draw(rGeom,rTraits,refPoint,!m_invert);
            else 
                node.draw(rGeom,rTraits,refPoint,m_invert);
            refPoint.x+=1;
        }
        for(size_t i= 0; i < m_margin; i++ )
        {
            node.draw(rGeom,rTraits,refPoint, m_invert);
            refPoint.x+=1;
        }
        refPoint.y-=1;
    }
    for(size_t i= 0; i < m_margin; i++ )
    {
        refPoint.x = 0;
        for(size_t j= 0; j < realwidth; j++ )
        {
            node.draw(rGeom,rTraits,refPoint, m_invert);
            refPoint.x+=1;
        }
        refPoint.y-=1;
    }
    rGeom.popModelTransform();
    return eOk;
}

Acad::ErrorStatus CExtQRCode::generateEmptyQRCode( AcGiWorldDraw *mode )
{
    m_version = -1;
    size_t x = 0, y = 0;
    const size_t width = 25;
    const size_t realwidth = (width + m_margin * 2);

    CQRCodeNode node;
    const double sc = m_scale/realwidth;
    AcGePoint3d refPoint (0.0, realwidth-1,0.0);
    const AcGeVector3d yDir = -1 * m_direction.crossProduct(m_normal).normalize();
    AcGeMatrix3d xform; xform.setCoordSystem(m_position,m_direction*sc,yDir*sc,m_normal*sc);

    AcGiSubEntityTraits& rTraits = mode->subEntityTraits();
    AcGiWorldGeometry& rGeom = mode->geometry();
    rGeom.pushModelTransform(xform);

    bool flipColor = false;
    for(y = 0; y < realwidth; y++) 
    {
        refPoint.x = 0;
        for(x=0; x<realwidth; x++) 
        {
            node.draw(rGeom,rTraits,refPoint,flipColor);
            if(flipColor)
                flipColor = false;
            else
               flipColor = true;
            refPoint.x+=1;
        }
        refPoint.y-=1;
    }
    mode->geometry().popModelTransform();
    return eOk;
}
