// QRCodeDialog.cpp : implementation file
//

#include "stdafx.h"
#include "QRCodeDialog.h"
#include "afxdialogex.h"
#include "CExtQRCode.h"


// QRCodeDialog dialog

IMPLEMENT_DYNAMIC(QRCodeDialog, CDialog)

    QRCodeDialog::QRCodeDialog(CExtQRCode *pCode, CWnd* pParent /*=NULL*/)
    : CDialog(QRCodeDialog::IDD, pParent), m_pCode(pCode)
{
    chkptr(m_pCode);
}

QRCodeDialog::~QRCodeDialog()
{
}

void QRCodeDialog::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_EDIT_CONTENT, m_ceditContents);
    DDX_Control(pDX, IDC_EDIT_SCALE, m_ceditScale);
    DDX_Control(pDX, IDC_EDIT_MARGIN, m_ceditMargin);
    DDX_Control(pDX, IDC_CHECK_INVERT, m_buttonInvert);
    DDX_Control(pDX, IDC_CHECK_CASE, m_buttonCaseSensitive);
}

BOOL QRCodeDialog::OnInitDialog()
{
    CDialog::OnInitDialog();
    if(chkptr(m_pCode))
    {
        m_ceditContents.SetWindowText(m_pCode->contents());
        m_buttonInvert.SetCheck(m_pCode->invert()?1:0);
        m_buttonCaseSensitive.SetCheck(m_pCode->isCaseSensitive()?1:0);

        CString str;
        str.Format(_T("%g"),m_pCode->scale());
        m_ceditScale.SetWindowText(str);

        str.Format(_T("%ld"),m_pCode->margin());
        m_ceditMargin.SetWindowText(str);

        switch(m_pCode->ecLevel())
        {
        case CExtQRCode::kQR_ECLEVEL_L:
            {
                CButton* pButton =(CButton*)GetDlgItem(IDC_RADIO_L);
                pButton->SetCheck(1);
            }
            break;
        case CExtQRCode::kQR_ECLEVEL_M:
            {
                CButton* pButton =(CButton*)GetDlgItem(IDC_RADIO_M);
                pButton->SetCheck(1);
            }
            break;
        case CExtQRCode::kQR_ECLEVEL_Q:
            {
                CButton* pButton =(CButton*)GetDlgItem(IDC_RADIO_Q);
                pButton->SetCheck(1);
            }
            break;
        case CExtQRCode::kQR_ECLEVEL_H:
            {
                CButton* pButton =(CButton*)GetDlgItem(IDC_RADIO_H);
                pButton->SetCheck(1);
            }
            break;
        }

        GotoDlgCtrl(GetDlgItem(IDC_EDIT_CONTENT)); 
    }
    return FALSE;
}

BEGIN_MESSAGE_MAP(QRCodeDialog, CDialog)
    ON_EN_CHANGE(IDC_EDIT_CONTENT, &QRCodeDialog::OnEnChangeEditContent)
    ON_EN_CHANGE(IDC_EDIT_SCALE, &QRCodeDialog::OnEnChangeEditScale)
    ON_EN_CHANGE(IDC_EDIT_MARGIN, &QRCodeDialog::OnEnChangeEditMargin)
    ON_BN_CLICKED(IDC_CHECK_INVERT, &QRCodeDialog::OnBnClickedCheckInvert)
    ON_BN_CLICKED(IDC_CHECK_CASE, &QRCodeDialog::OnBnClickedCheckCase)
    ON_BN_CLICKED(IDC_RADIO_L, &QRCodeDialog::OnBnClickedRadioL)
    ON_BN_CLICKED(IDC_RADIO_M, &QRCodeDialog::OnBnClickedRadioM)
    ON_BN_CLICKED(IDC_RADIO_Q, &QRCodeDialog::OnBnClickedRadioQ)
    ON_BN_CLICKED(IDC_RADIO_H, &QRCodeDialog::OnBnClickedRadioH)
END_MESSAGE_MAP()


// QRCodeDialog message handlers
void QRCodeDialog::OnEnChangeEditContent()
{
    if(chkptr(m_pCode))
    {
        CString txt;
        m_ceditContents.GetWindowText(txt);

        bool hasProtocol = false;
        if(txt.Find(_T("http"))!=-1)
            hasProtocol = true;
        if(txt.Find(_T("ftp"))!=-1)
            hasProtocol = true;
        if(hasProtocol && m_buttonCaseSensitive.GetCheck() == 0)
        {
            m_buttonCaseSensitive.SetCheck(1);
            OnBnClickedCheckCase();
        }
        m_pCode->setContents(txt);
    }
}

void QRCodeDialog::OnEnChangeEditScale()
{
    if(chkptr(m_pCode))
    {
        CString str;
        m_ceditScale.GetWindowText(str);
        double val = _tstof(str);
        if(val>0)
        {
            m_pCode->setScale(val);
        }
        else
        {
            acutPrintf(_T("\nScale cannot be 0"));
        }
    }
}

void QRCodeDialog::OnEnChangeEditMargin()
{
    if(chkptr(m_pCode))
    {
        CString str;
        m_ceditMargin.GetWindowText(str);
        int val = _tstoi(str);
        m_pCode->setMargin(val);
    }
}

void QRCodeDialog::OnBnClickedCheckInvert()
{
    if(chkptr(m_pCode))
        m_pCode->setInvert(m_buttonInvert.GetCheck() != 0);
}

void QRCodeDialog::OnBnClickedCheckCase()
{
    if(chkptr(m_pCode))
        m_pCode->setCaseSensitive(m_buttonCaseSensitive.GetCheck() != 0);
}

void QRCodeDialog::OnBnClickedRadioL()
{
    if(chkptr(m_pCode))
        m_pCode->setEcLevel(CExtQRCode::kQR_ECLEVEL_L);
}

void QRCodeDialog::OnBnClickedRadioM()
{
    if(chkptr(m_pCode))
        m_pCode->setEcLevel(CExtQRCode::kQR_ECLEVEL_M);
}

void QRCodeDialog::OnBnClickedRadioQ()
{
    if(chkptr(m_pCode))
        m_pCode->setEcLevel(CExtQRCode::kQR_ECLEVEL_Q);
}

void QRCodeDialog::OnBnClickedRadioH()
{
    if(chkptr(m_pCode))
        m_pCode->setEcLevel(CExtQRCode::kQR_ECLEVEL_H);
}

bool QRCodeDialog::chkptr( const CExtQRCode *pCode )
{
    if(pCode != NULL && pCode->isWriteEnabled())
        return true;
    else
        acutPrintf(_T("\nError in QRCodeDialog::chkptr"));
    return false;
}
