#include "StdAfx.h"
#include "CExtQRCode.h"
#include "QRCodeOPMClientExtension.h"

#ifndef BRX_APP  // // dummy implemented for ARX build
bool registerQRCodeOPMExtension()
{
    return false;
}
bool unregisterQRCodeOPMExtension()
{
    return false;
}
#endif // !BRX_APP

#ifdef BRX_APP  // not available with ARX

static CQRCodeOPMClientExtension* pQRCodeOPMClientExtension = NULL;

bool clientCallbackFunction(const AcDbObjectIdArray& pickSet,
    AcOPMPropertyId propertyId, const AcString& propertyName,
    const AcString& childName, AcOPMVariant& value);


bool registerQRCodeOPMExtension()
{
    pQRCodeOPMClientExtension = new CQRCodeOPMClientExtension();

    // register the properties for target entity type
    AcOpmResult res = acRegisterEntityExtension(pQRCodeOPMClientExtension, CExtQRCode::desc(), true);

    return (res == opmNoError);
}

bool unregisterQRCodeOPMExtension()
{
    //bool res = acUnregisterEntityExtension(pQRCodeOPMClientExtension, CExtQRCode::desc());
    bool res = acRemoveOPMExtension(pQRCodeOPMClientExtension);
    delete pQRCodeOPMClientExtension, pQRCodeOPMClientExtension = NULL;

    return true;
}


CQRCodeOPMClientExtension::CQRCodeOPMClientExtension(void)
{
}


CQRCodeOPMClientExtension::~CQRCodeOPMClientExtension(void)
{
    bool res = acRemoveOPMExtension(pQRCodeOPMClientExtension);
}

bool CQRCodeOPMClientExtension::getPropertyMap( const AcRxClass* entityClass, AcOPMPropertyArray& properties ) const
{
    properties.append(AcOPMPropertyEntry(_T("QRCode Stetings"), _T("Contents"),AcOPMPROP_FirstUserProp , opmTypeString));
    properties.append(AcOPMPropertyEntry(_T("QRCode Stetings"), _T("Scale"),   AcOPMPROP_FirstUserProp+1  , opmTypeDouble));
    properties.append(AcOPMPropertyEntry(_T("QRCode Stetings"), _T("Margin"),  AcOPMPROP_FirstUserProp+2 , opmTypeInteger));
    properties.append(AcOPMPropertyEntry(_T("QRCode Stetings"), _T("Invert"),  AcOPMPROP_FirstUserProp+3 , opmTypeBool));
    properties.append(AcOPMPropertyEntry(_T("QRCode Stetings"), _T("Case sensitive"),  AcOPMPROP_FirstUserProp+4 , opmTypeBool));
    properties.append(AcOPMPropertyEntry(_T("QRCode Stetings"), _T("Error correction"),  AcOPMPROP_FirstUserProp+5 , opmTypeEnum));
    properties.last().m_predefinedStrings.append(_T("7%"));
    properties.last().m_predefinedStrings.append(_T("15%"));
    properties.last().m_predefinedStrings.append(_T("25%"));
    properties.last().m_predefinedStrings.append(_T("30%"));
    properties.append(AcOPMPropertyEntry(_T("QRCode Stetings"), _T("Version"),  AcOPMPROP_FirstUserProp+6 , opmTypeInteger));
    return true;
}

bool CQRCodeOPMClientExtension::getObjectName( const AcDbEntity* entity, AcString& name ) const
{
    return false;
}

bool CQRCodeOPMClientExtension::getCategoryName( const AcString& category, AcString& name ) const
{
    return false;
}

bool CQRCodeOPMClientExtension::getCategoryName( AcOPMCategoryId category, AcString& name ) const
{
    return false;
}

bool CQRCodeOPMClientExtension::getPropertyName( const AcDbEntity* entity, AcOPMPropertyId propertyId, const AcString& childName, AcString& name ) const
{
    return false;
}

bool CQRCodeOPMClientExtension::getPropertyDescription( const AcDbEntity* entity, AcOPMPropertyId propertyId, const AcString& childName, AcString& description ) const
{
    switch (propertyId)
    {
    case AcOPMPROP_FirstUserProp :     
        description = _T("Contents String");
        return true;
    case AcOPMPROP_FirstUserProp+1 :     
        description = _T("Scale");
        return true;
    case AcOPMPROP_FirstUserProp+2 :     
        description = _T("Margin");
        return true;
    case AcOPMPROP_FirstUserProp+3:     
        description = _T("Invert");
        return true;
    case AcOPMPROP_FirstUserProp+4:     
        description = _T("Case sensistive");
        return true;
    case AcOPMPROP_FirstUserProp+5:     
        description = _T("Error correction");
        return true;
    case AcOPMPROP_FirstUserProp+6:     
        description = _T("Version");
        return true;
    }
    return true;
}

bool CQRCodeOPMClientExtension::getPropertyEnabled( const AcDbEntity* entity, AcOPMPropertyId propertyId, const AcString& childName, bool& enabled, bool& hideDisabled ) const
{
    return true;
}

bool CQRCodeOPMClientExtension::getPropertyReadOnly( const AcDbEntity* entity, AcOPMPropertyId propertyId, const AcString& childName, bool& readOnly ) const
{
    switch (propertyId)
    {
    case AcOPMPROP_FirstUserProp+6:
        {
            readOnly = true;
            return true;
        }
    }
    return false;
}

bool CQRCodeOPMClientExtension::getPredefinedStrings( const AcDbEntity* entity, AcOPMPropertyId propertyId, const AcString& childName, AcStringArray& predefinedStrings ) const
{
    return false;
}

bool CQRCodeOPMClientExtension::getPropertyValue( const AcDbEntity* entity, AcOPMPropertyId propertyId, const AcString& childName, AcOPMVariant& value ) const
{
    const CExtQRCode *pCode = static_cast<const CExtQRCode*>(entity);
    switch (propertyId)
    {
    case AcOPMPROP_FirstUserProp : 
        {
            value = pCode->contents();
            return true;
        }
    case AcOPMPROP_FirstUserProp+1 : 
        {
            value = pCode->scale();
            return true;
        }
    case AcOPMPROP_FirstUserProp+2 : 
        {
            value = pCode->margin();
            return true;
        }
    case AcOPMPROP_FirstUserProp+3 : 
        {
            value = pCode->invert();
            return true;
        }
    case AcOPMPROP_FirstUserProp+4 : 
        {
            value = pCode->isCaseSensitive();
            return true;
        }
    case AcOPMPROP_FirstUserProp+5 : 
        {
            value = pCode->ecLevel();
            return true;
        }
    case AcOPMPROP_FirstUserProp+6 : 
        {
            value = pCode->version();
            return true;
        }
    }
    return false;
}

bool CQRCodeOPMClientExtension::setPropertyValue( AcDbEntity* entity, AcOPMPropertyId propertyId, const AcString& childName, const AcOPMVariant& value )
{
    CExtQRCode *pCode = static_cast<CExtQRCode*>(entity);
    switch (propertyId)
    {
    case AcOPMPROP_FirstUserProp :  
        {
            AcString val;
            if(value.get(val))
            {
                pCode->setContents(val);
                return true;
            }
        }
        break;
    case AcOPMPROP_FirstUserProp+1 : 
        {
            double val;
            if(value.get(val))
            {
                pCode->setScale(val);
                return true;
            }
        }
        break;
    case AcOPMPROP_FirstUserProp+2 : 
        {
            int val;
            if(value.get(val))
            {
                pCode->setMargin(val);
                return true;
            }
        }
        break;
    case AcOPMPROP_FirstUserProp+3 : 
        {
            bool val;
            if(value.get(val))
            {
                pCode->setInvert(val);
                return true;
            }
        }
        break;
    case AcOPMPROP_FirstUserProp+4 : 
        {
            bool val;
            if(value.get(val))
            {
                pCode->setCaseSensitive(val);
                return true;
            }
        }
        break;
    case AcOPMPROP_FirstUserProp+5 : 
        {
            int val;
            if(value.get(val))
            {
                pCode->setEcLevel((CExtQRCode::QRECLevel)val);
                return true;
            }
        }
        break;
    case AcOPMPROP_FirstUserProp+6 : 
        {
            return true;
        }
        break;
    }
    return false;
}


#endif
