// (C) Copyright 2002-2007 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

//-----------------------------------------------------------------------------
//----- acrxEntryPoint.cpp
//-----------------------------------------------------------------------------
#include "StdAfx.h"
#include "resource.h"
#include "QRCodeJig.h"
#include "QRCodeDialog.h"
#include "QRCodeDoubleClickEdit.h"
#include "QRCodeOPMClientExtension.h"

//-----------------------------------------------------------------------------
#define szRDS _RXST("")

//-----------------------------------------------------------------------------
class CQRCodeAppApp : public AcRxArxApp 
{
private:
    CQRCodeDoubleClickEdit m_pEdit;

public:
    CQRCodeAppApp () : AcRxArxApp () 
    {
    }

    virtual AcRx::AppRetCode On_kInitAppMsg (void *pkt) 
    {
        AcRx::AppRetCode retCode =AcRxArxApp::On_kInitAppMsg (pkt) ;
        acrxDynamicLinker->lockApplication(pkt);
        registerQRCodeOPMExtension();
        CExtQRCode::desc()->addX(AcDbDoubleClickEdit::desc(),&m_pEdit);
        return (retCode) ;
    }

    virtual AcRx::AppRetCode On_kUnloadAppMsg (void *pkt) 
    {
        unregisterQRCodeOPMExtension();
        CExtQRCode::desc()->delX(AcDbDoubleClickEdit::desc());
        AcRx::AppRetCode retCode =AcRxArxApp::On_kUnloadAppMsg (pkt) ;
        return (retCode) ;
    }

    virtual void RegisterServerComponents ()
    {
    }

    static AcDbObjectId postToDatabase(AcDbEntity *pEnt)
    {
        AcDbObjectId Id;
        AcDbDatabase *pDb = acdbHostApplicationServices()->workingDatabase();
        AcDbBlockTableRecordPointer pCurSpace(pDb->currentSpaceId(),AcDb::kForWrite);
        if(pCurSpace.openStatus() == eOk)
        {
            pCurSpace->appendAcDbEntity(Id, pEnt);
        }
        return Id;
    }

    static bool filedia()
    {
        resbuf buf;
        if (acedGetVar(_T("FILEDIA"), &buf) == RTNORM)
            return buf.resval.rint == 1 ? true : false;
        return true;
    }

    static AcString getString()
    {
        AcString val;
        ACHAR* result = nullptr;
        if (acedGetFullString(1, _T("\nEnter a string: "), result) == RTNORM && result != nullptr)
        {
            val = result;
            acutDelString(result);
        }
        return val;
    }

    static void QRCodeApp_QRCode(void)
    {
        AcGeMatrix3d ucs;
        AcApDocument *pDoc=acDocManager->curDocument();
        acDocManager->lockDocument(pDoc,AcAp::kWrite); 
        std::unique_ptr<CExtQRCode, DBObjectDeleter> pCode (new CExtQRCode());
        if(acedGetCurrentUCS(ucs) == eOk)
        {
            pCode->transformBy(ucs);
            QRCodeJig jig(pCode.get());
            if(jig.getPoint(_T("\nLocation of QRcode: ")))
            {
                bool post = false;
                if (filedia())
                {
                    QRCodeDialog dlg(pCode.get(), AcUiMainWindow());
                    if (IDOK == dlg.DoModal())
                        post = true;
                }
                else
                {
                    pCode->setContents(getString());
                    post = true;
                }
                if (post)
                {
                    postToDatabase(pCode.get());
                }
                
            }
        }
        acDocManager->unlockDocument(pDoc);
    }

} ;

//-----------------------------------------------------------------------------
IMPLEMENT_ARX_ENTRYPOINT(CQRCodeAppApp)

ACED_ARXCOMMAND_ENTRY_AUTO(CQRCodeAppApp, QRCodeApp, _QRCode, QRCode, ACRX_CMD_TRANSPARENT, NULL)
