// Copyright (C) Menhirs NV. All rights reserved.
// License to copy and modify this file for usage in conjunction
// with Autodesk ARX API is explicitely expressed here, as long as the
// copyright statement remains unchanged !
// Please copy this file to your Autodesk ARX API or project folder(s).

// this header profides following features :
// provides name mangling for ARX <= 2009 vs. ARX >= 2010 vs. BRX 12/13
// ("subWorldDraw" vs. "worldDraw"); it wraps both "old" and "new" naming convention to
// the correct one, depending on ARX/BRX build version

// You must have "BRX_APP" defined for BRX builds (without BRX_APP, it is an ARX build)!


#ifdef BRX_APP  // BRX builds
  #include "brx_version.h"
#else           // ARX builds
  #include "arx_version.h"
#endif // BRX_APP


#if defined(__ARX)

  #if (__ARXVER >= 2010)

      // for ARX >= 2010 enforce "new" naming convention (for given "old" names)
      // AcGiDrawable
      #define setAttributes                subSetAttributes
      #define worldDraw                    subWorldDraw
      #define viewportDraw                 subViewportDraw
      #define viewportDrawLogicalFlags     subViewportDrawLogicalFlags

      // AcDbObject
      #define deepClone                    subDeepClone
      #define wblockClone                  subWblockClone
      #define getClassID                   subGetClassID
      #define getGeomExtents               subGetGeomExtents
      #define addSubentPaths               subAddSubentPaths
      #define deleteSubentPaths            subDeleteSubentPaths
      #define getSubentClassId             subGetSubentClassId
      #define transformSubentPathsBy       subTransformSubentPathsBy
      #define getGripPointsAtSubentPath    subGetGripPointsAtSubentPath
      #define moveGripPointsAtSubentPaths  subMoveGripPointsAtSubentPaths
      #define getSubentPathGeomExtents     subGetSubentPathGeomExtents
      #define getSubentPathsAtGsMarker     subGetSubentPathsAtGsMarker
      #define getGsMarkersAtSubentPath     subGetGsMarkersAtSubentPath
      #define highlight                    subHighlight
      #define unhighlight                  subUnhighlight
      #define subentPtr                    subSubentPtr
      #define getOsnapPoints               subGetOsnapPoints
      #define isContentSnappable           subIsContentSnappable
      #define getGripPoints                subGetGripPoints
      #define moveGripPointsAt             subMoveGripPointsAt
      #define getStretchPoints             subGetStretchPoints
      #define moveStretchPointsAt          subMoveStretchPointsAt
      #define intersectWith                subIntersectWith
      #define list                         subList
      #define getTransformedCopy           subGetTransformedCopy
      #define explode                      subExplode
      #define getCompoundObjectTransform   subGetCompoundObjectTransform
      #define cloneMeForDragging           subCloneMeForDragging
      #define hideMeForDragging            subHideMeForDragging
      #define gripStatus                   subGripStatus
      #define subentGripStatus             subSubentGripStatus
      #define getGripEntityUCS             subGetGripEntityUCS

      // these functions exist as both "subXXX()" and "XXX()" functions, so name mapping
      // does not work here; using uppercase placeholder names
      #define TransformBy                  subTransformBy
      #define SetVisibility                subSetVisibility

  #else  // ARX 2009 and before

      // for ARX <= 2009 enforce "old" naming convention (for given "new" names)
      // AcGiDrawable
      #define subSetAttributes                 setAttributes
      #define subWorldDraw                     worldDraw
      #define subViewportDraw                  viewportDraw
      #define subViewportDrawLogicalFlags      viewportDrawLogicalFlags

      // AcDbObject
      #define subDeepClone                     deepClone
      #define subWblockClone                   wblockClone
      #define subGetClassID                    getClassID
      #define subSetVisibility                 setVisibility
      #define subGetGeomExtents                getGeomExtents
      #define subAddSubentPaths                addSubentPaths
      #define subDeleteSubentPaths             deleteSubentPaths
      #define subGetSubentClassId              getSubentClassId
      #define subTransformSubentPathsBy        transformSubentPathsBy
      #define subGetGripPointsAtSubentPath     getGripPointsAtSubentPath
      #define subMoveGripPointsAtSubentPaths   moveGripPointsAtSubentPaths
      #define subGetSubentPathGeomExtents      getSubentPathGeomExtents
      #define subGetSubentPathsAtGsMarker      getSubentPathsAtGsMarker
      #define subGetGsMarkersAtSubentPath      getGsMarkersAtSubentPath
      #define subHighlight                     highlight
      #define subUnhighlight                   unhighlight
      #define subSubentPtr                     subentPtr
      #define subGetOsnapPoints                getOsnapPoints
      #define subIsContentSnappable            isContentSnappable
      #define subGetGripPoints                 getGripPoints
      #define subMoveGripPointsAt              moveGripPointsAt
      #define subGetStretchPoints              getStretchPoints
      #define subMoveStretchPointsAt           moveStretchPointsAt
      #define subIntersectWith                 intersectWith
      #define subList                          list
      #define subGetTransformedCopy            getTransformedCopy
      #define subExplode                       explode
      #define subGetCompoundObjectTransform    getCompoundObjectTransform
      #define subCloneMeForDragging            cloneMeForDragging
      #define subHideMeForDragging             hideMeForDragging
      #define subGripStatus                    gripStatus
      #define subSubentGripStatus              subentGripStatus
      #define subGetGripEntityUCS              getGripEntityUCS
      #define subTransformBy                   transformBy

      // these functions exist as both "subXXX()" and "XXX()" functions, so name mapping
      // does not work here; using uppercase placeholder names
      #define TransformBy                      transformBy
      #define SetVisibility                    setVisibility

  #endif

#endif // __ARX

#if defined(__BRX) // identical to "ARX2009 and before"
  // for BRX builds, enforce "old" naming convention (for given "new" names)

  // AcGiDrawable
  #define subSetAttributes                 setAttributes
  #define subWorldDraw                     worldDraw
  #define subViewportDraw                  viewportDraw
  #define subViewportDrawLogicalFlags      viewportDrawLogicalFlags

  // AcDbObject
  #define subDeepClone                     deepClone
  #define subWblockClone                   wblockClone
  #define subGetClassID                    getClassID
  #define subSetVisibility                 setVisibility
  #define subGetGeomExtents                getGeomExtents
  #define subAddSubentPaths                addSubentPaths
  #define subDeleteSubentPaths             deleteSubentPaths
  #define subGetSubentClassId              getSubentClassId
  #define subTransformSubentPathsBy        transformSubentPathsBy
  #define subGetGripPointsAtSubentPath     getGripPointsAtSubentPath
  #define subMoveGripPointsAtSubentPaths   moveGripPointsAtSubentPaths
  #define subGetSubentPathGeomExtents      getSubentPathGeomExtents
  #define subGetSubentPathsAtGsMarker      getSubentPathsAtGsMarker
  #define subGetGsMarkersAtSubentPath      getGsMarkersAtSubentPath
  #define subHighlight                     highlight
  #define subUnhighlight                   unhighlight
  #define subSubentPtr                     subentPtr
  #define subGetOsnapPoints                getOsnapPoints
  #define subIsContentSnappable            isContentSnappable
  #define subGetGripPoints                 getGripPoints
  #define subMoveGripPointsAt              moveGripPointsAt
  #define subGetStretchPoints              getStretchPoints
  #define subMoveStretchPointsAt           moveStretchPointsAt
  #define subIntersectWith                 intersectWith
  #define subList                          list
  #define subGetTransformedCopy            getTransformedCopy
  #define subExplode                       explode
  #define subGetCompoundObjectTransform    getCompoundObjectTransform
  #define subCloneMeForDragging            cloneMeForDragging
  #define subHideMeForDragging             hideMeForDragging
  #define subGripStatus                    gripStatus
  #define subSubentGripStatus              subentGripStatus
  #define subGetGripEntityUCS              getGripEntityUCS
  #define subTransformBy                   transformBy

  // these functions exist as both "subXXX()" and "XXX()" functions, so name mapping
  // does not work here; using uppercase placeholder names
  #define TransformBy                      transformBy
  #define SetVisibility                    setVisibility

#endif // __BRX
